/*
 * Copyright (c) 2017 Nick Mobile as an unpublished work. Neither
 * this material nor any portion thereof may be copied or distributed
 * without the express written consent of Nick Mobile.
 * This material also contains proprietary and confidential information
 * of 2017 Nick Mobile and its suppliers, and may not be used by or
 * disclosed to any person, in whole or in part, without the prior written
 * consent of 2017 Nick Mobile.
 */

(function() {
    Timer = function() {
        this.reset();
        return this;
    };
    Timer.prototype = {
        currentTimeInMillis: 0,
        lastTimeInMillis: 0,
        elapsedTimeInMillis: 0,
        elapsedTimeInSeconds: 0,
        accumTimeInMillis: 0,
        reset: function() {
            this.currentTimeInMillis = this.lastTimeInMillis = new Date().getTime();
            this.accumTimeInMillis = this.elapsedTimeInMillis = this.elapsedTimeInSeconds = 0;
        },
        update: function() {
            this.currentTimeInMillis = new Date().getTime();
            this.elapsedTimeInMillis = this.currentTimeInMillis - this.lastTimeInMillis;
            this.elapsedTimeInSeconds = this.elapsedTimeInMillis / 1000.0;
            this.lastTimeInMillis = this.currentTimeInMillis;
            this.accumTimeInMillis += this.elapsedTimeInMillis;
        }
    };
    FPSCounter = function() {
        this.timer = new Timer();
        this.reset();
        return this;
    };
    FPSCounter.prototype = {
        timer: 0,
        fps: 0,
        averageFPS: 0,
        reset: function() {
            this.timer.reset();
            this.fps = 0;
            this.averageFPS = 0;
        },
        update: function() {
            this.timer.update();
            this.fps++;
            if (this.timer.accumTimeInMillis >= 1000) {
                this.averageFPS = ((1000 * this.fps / this.timer.accumTimeInMillis * 100) | 0) / 100;
                this.fps = 0;
                this.timer.reset();
            }
        }
    };

    function drawText(canvasContext, text, textAlign, x, y, fontSize) {
        canvasContext.font = fontSize + 'px Arial';
        canvasContext.textAlign = textAlign;
        canvasContext.textBaseline = 'middle';
        canvasContext.fillStyle = 'white';
        canvasContext.fillText(text, x, y);
    }

    function setupFPS() {
        var fpsCounter = new FPSCounter();
        var fontSize = 20;
        var canvas = document.createElement('canvas');
        canvas.width = 2 * fontSize;
        canvas.height = 2 * fontSize;
        document.body.appendChild(canvas);
        var canvasContext = canvas.getContext('2d');
        var left = leftOrRight ? 0 : window.innerWidth - canvas.width;
        var top = topOrBottom ? 0 : window.innerHeight - canvas.height;
        canvas.style.left = left;
        canvas.style.top = top;
        canvas.style.position = 'absolute';
        var textAlign = 'center';
        var fpsIntervalId = setInterval(function() {
            canvasContext.fillStyle = 'red';
            canvasContext.fillRect(0, 0, canvas.width, canvas.height);
            fpsCounter.update();

            // Align text to center
            drawText(canvasContext,
                '' + Math.round(fpsCounter.averageFPS),
                textAlign,
                canvas.width >> 1,
                (canvas.height / 2),
                fontSize);

        }, 1000 / 60);
    }

    var useWindowLoad = false;
    var useTimeout = true;
    var timeout = 5000;
    var textColor = 'white';
    var topOrBottom = true;
    var leftOrRight = false;
    var timeIntervalOrRequestAnimationFrame = false;
    if (useWindowLoad) {
        window.addEventListener('load', function() {
            if (useTimeout) {
                setTimeout(setupFPS, timeout);
            } else {
                setupFPS();
            }
        });
    } else {
        if (useTimeout) {
            setTimeout(setupFPS, timeout);
        } else {
            setupFPS();
        }
    }
})();